/* 
   $Id: gtksgctree.c,v 1.6 2001/12/20 07:39:29 shibajun Exp $

   Copyright (C) 2001-2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of sylpheed.

   sylpheed is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   sylpheed is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with sylpheed; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */


#include <gtk/gtkctree.h>
#include "gtksgctree.h"

static void gtk_sgctree_class_init    (GtkSGCTreeClass   *klass);
static void gtk_sgctree_init          (GtkSGCTree        *ctree);

static gint gtk_sgctree_button_press (GtkWidget      *widget,
				      GdkEventButton *event) ;

enum
  {
    TREE_OPEN_ROW,
    LAST_SIGNAL
  } ;

static guint sgctree_signals[LAST_SIGNAL] ;
static GtkCTreeClass *parent_class = NULL ;

guint
gtk_sgctree_get_type (void)
{
  static guint sgctree_type = 0;

  if (!sgctree_type)
    {
      static const GtkTypeInfo sgctree_info =
      {
	"GtkSGCTree",
	sizeof (GtkSGCTree),
	sizeof (GtkSGCTreeClass),
	(GtkClassInitFunc) gtk_sgctree_class_init,
	(GtkObjectInitFunc) gtk_sgctree_init,
	/* reserved_1 */ NULL,
        /* reserved_2 */ NULL,
        (GtkClassInitFunc) NULL,
      };

      sgctree_type = gtk_type_unique (gtk_ctree_get_type (), &sgctree_info);
    }

  return sgctree_type;
}

static void
gtk_sgctree_class_init (GtkSGCTreeClass *klass)
{
  GtkWidgetClass *widget_class;
  GtkObjectClass *object_class;

  widget_class = (GtkWidgetClass*) klass;
  object_class = (GtkObjectClass*) klass;

  widget_class->button_press_event = gtk_sgctree_button_press;

  parent_class = gtk_type_class (gtk_ctree_get_type()) ;

  sgctree_signals[TREE_OPEN_ROW] = 
    gtk_signal_new ("tree_open_row",
		    GTK_RUN_FIRST,
		    object_class->type,
		    GTK_SIGNAL_OFFSET (GtkSGCTreeClass, tree_open_row),
		    gtk_marshal_NONE__POINTER,
		    GTK_TYPE_NONE, 1, GTK_TYPE_CTREE_NODE);

  gtk_object_class_add_signals (object_class, sgctree_signals, LAST_SIGNAL) ;
}

static void
gtk_sgctree_init (GtkSGCTree *sgctree)
{
}


static gint
gtk_sgctree_button_press (GtkWidget      *widget,
			GdkEventButton *event)
{
  GtkCTree *ctree;
  GtkCList *clist;
  gint button_actions;

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_CTREE (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  ctree = GTK_CTREE (widget);
  clist = GTK_CLIST (widget);

  button_actions = clist->button_actions[event->button - 1];

  if (button_actions == GTK_BUTTON_IGNORED)
    return FALSE;
  if (event->window == clist->clist_window)
    {
      GtkCTreeNode *work;
      gint x;
      gint y;
      gint row;
      gint column;
      static gint previous_row ;

      x = event->x;
      y = event->y;

      if (!gtk_clist_get_selection_info (clist, x, y, &row, &column))
	return FALSE;

      work = GTK_CTREE_NODE (g_list_nth (clist->row_list, row));
	  
#if 0
      g_print ("\nbutton_actions & GTK_BUTTON_EXPANDS = %x\n",
	       button_actions & GTK_BUTTON_EXPANDS) ;
      g_print ("GTK_CTREE_ROW (work)->childrena = %x\n",
	       GTK_CTREE_ROW (work)->children) ;
      g_print ("GTK_CTREE_ROW (work)->is_leaf = %x\n",
	       GTK_CTREE_ROW (work)->is_leaf) ;
      g_print ("event->type == GDK_2BUTTON_PRESS = %x\n",
	       (event->type == GDK_2BUTTON_PRESS)) ;
      g_print ("gtk_ctree_is_hot_spot (ctree, x, y) = %x\n\n",
	       gtk_ctree_is_hot_spot (ctree, x, y)) ;
#endif
      if (button_actions & GTK_BUTTON_EXPANDS &&
	  (!GTK_CTREE_ROW (work)->children  && 
	   ((event->type == GDK_BUTTON_PRESS) &&
	    (row == previous_row))))
	{
#if 0
	  g_print ("gtksgctree.c:single click on the leaf") ;
	  g_print ("previous_row = %d, row = %d\n", previous_row, row) ;
#endif
	  gtk_signal_emit (GTK_OBJECT(widget), sgctree_signals[TREE_OPEN_ROW]) ;
#if 0
	  if (GTK_CTREE_ROW (work)->expanded)
	    gtk_ctree_collapse (ctree, work);
	  else
	    gtk_ctree_expand (ctree, work);
#endif
	  return FALSE;
	}
#if 0
      if (button_actions & GTK_BUTTON_EXPANDS &&
	  (GTK_CTREE_ROW (work)->children && !GTK_CTREE_ROW (work)->is_leaf  &&
	   (event->type == GDK_2BUTTON_PRESS ||
	    ctree_is_hot_spot (ctree, work, row, x, y))))
	{
	  g_print ("double click on the hotspot\n") ;
	  if (GTK_CTREE_ROW (work)->expanded)
	    gtk_ctree_collapse (ctree, work);
	  else
	    gtk_ctree_expand (ctree, work);

	  return FALSE;
	}
#endif
      previous_row = row ;
    }
  return GTK_WIDGET_CLASS (parent_class)->button_press_event (widget, event);
}

GtkWidget *gtk_sgctree_new_with_titles (gint columns, 
				       gint tree_column, 
				       gchar *titles[])
{
	GtkSGCTree* sgctree;

	sgctree = gtk_type_new (gtk_sgctree_get_type ());
	gtk_ctree_construct (GTK_CTREE (sgctree), columns, tree_column, titles);
	gtk_clist_set_selection_mode(GTK_CLIST(sgctree), GTK_SELECTION_EXTENDED);

	return GTK_WIDGET (sgctree);
}

